@@ -1,15 +1,23 @@
 .onLoad <- function(libname, pkgname){ ## nocov start
     ## Setup RxODE.prefer.tbl
+    .Call(`_RxODE_setRstudio`, Sys.getenv("RSTUDIO")=="1")
     rxPermissive(respect=TRUE); ## need to call respect on the first time
     suppressMessages(.rxWinRtoolsPath())
 } ## nocov end
 
 .onAttach <- function(libname, pkgname){
+    .Call(`_RxODE_setRstudio`, Sys.getenv("RSTUDIO")=="1")
+    rxPermissive(respect=TRUE); ## need to call respect on the first time
     if (!.rxWinRtoolsPath()){
         packageStartupMessage("Rtools is not set up correctly!\n\nYou need a working Rtools installation for RxODE to work.\nYou can set up Rtools using the command 'rxWinSetup()'.\nThis will also set up Python and SymPy to run a bit faster than rSymPy.\n");
     }
 }
 
+.onUnload <- function (libpath) {
+    rxSolveFree();
+    library.dynam.unload("RxODE", libpath)
+}
+
 .rxTempDir0 <- NULL;
 .rxTempDir <- function(){
     if (is.null(getFromNamespace(".rxTempDir0", "RxODE"))){
